@@ -83,6 +83,18 @@ Char tolower_ascii(Char c) {
 template<>
 std::wstring::value_type FZ_PUBLIC_SYMBOL tolower_ascii(std::wstring::value_type c);
 
+/// \brief Converts ASCII lowercase characters to uppercase as if C-locale is used.
+template<typename Char>
+Char toupper_ascii(Char c) {
+	if (c >= 'a' && c <= 'z') {
+		return c + ('A' - 'a');
+	}
+	return c;
+}
+
+template<>
+std::wstring::value_type FZ_PUBLIC_SYMBOL toupper_ascii(std::wstring::value_type c);
+
 /** \brief tr_tolower_ascii does for strings what tolower_ascii does for individual characters
  */
  // Note: For UTF-8 strings it works on individual octets!
@@ -96,6 +108,16 @@ String str_tolower_ascii(String const& s)
 	return ret;
 }
 
+template<typename String>
+String str_toupper_ascii(String const& s)
+{
+	String ret = s;
+	for (auto& c : ret) {
+		c = toupper_ascii(c);
+	}
+	return ret;
+}
+
 /** \brief Converts from std::string in system encoding into std::wstring
  *
  * \return the converted string on success. On failure an empty string is returned.
@@ -168,17 +190,40 @@ std::string FZ_PUBLIC_SYMBOL to_utf8(std::wstring const& in);
  *
  * Example: '9' becomes 9, 'b' becomes 11, 'D' becomes 13
  *
- * Undefined output if input is not a valid hex digit.
+ * Returns -1 if input is not a valid hex digit.
  */
 template<typename Char>
 int hex_char_to_int(Char c)
 {
-	if (c >= 'a')
+	if (c >= 'a' && c <= 'z') {
 		return c - 'a' + 10;
-	if (c >= 'A')
+	}
+	if (c >= 'A' && c <= 'Z') {
 		return c - 'A' + 10;
-	else
+	}
+	else if (c >= '0' && c <= '9') {
 		return c - '0';
+	}
+	return -1;
+}
+
+template<typename String>
+std::vector<uint8_t> hex_decode(String const& in)
+{
+	std::vector<uint8_t> ret;
+	if (!(in.size() % 2)) {
+		ret.reserve(in.size() / 2);
+		for (size_t i = 0; i < in.size(); i += 2) {
+			int high = hex_char_to_int(in[i]);
+			int low = hex_char_to_int(in[i + 1]);
+			if (high == -1 || low == -1) {
+				return std::vector<uint8_t>();
+			}
+			ret.push_back(static_cast<uint8_t>((high << 4) + low));
+		}
+	}
+
+	return ret;
 }
 
 /** \brief Converts an integer to the corresponding lowercase hex digit
@@ -198,6 +243,20 @@ Char int_to_hex_char(int d)
 	}
 }
 
+template<typename String, typename InString, bool Lowercase = true>
+String hex_encode(InString const& data)
+{
+	static_assert(sizeof(typename InString::value_type) == 1, "Input must be a container of 8 bit values");
+	String ret;
+	ret.reserve(data.size() * 2);
+	for (auto const& c : data) {
+		ret.push_back(int_to_hex_char<typename String::value_type, Lowercase>(static_cast<unsigned char>(c) >> 4));
+		ret.push_back(int_to_hex_char<typename String::value_type, Lowercase>(static_cast<unsigned char>(c) & 0xf));
+	}
+
+	return ret;
+}
+
 /// Calls either fz::to_string or fz::to_wstring depending on the passed template argument
 template<typename String, typename Arg>
 inline auto toString(Arg&& arg) -> typename std::enable_if<std::is_same<String, std::string>::value, decltype(to_string(std::forward<Arg>(arg)))>::type
@@ -292,7 +351,7 @@ std::string FZ_PUBLIC_SYMBOL base64_decode(std::string const& in);
 
 // Converts string to integral type T. If string is not convertible, T() is returned.
 template<typename T, typename String>
-T to_integral(String const& s)
+T to_integral(String const& s, T const errorval = T())
 {
 	T ret{};
 
@@ -301,10 +360,14 @@ T to_integral(String const& s)
 		++it;
 	}
 
+	if (it == s.cend()) {
+		return errorval;
+	}
+
 	for (; it != s.cend(); ++it) {
 		auto const& c = *it;
 		if (c < '0' || c > '9') {
-			return T();
+			return errorval;
 		}
 		ret *= 10;
 		ret += c - '0';
@@ -318,6 +381,40 @@ T to_integral(String const& s)
 	}
 }
 
+/// \brief Returns true iff the string only has characters in the 7-bit ASCII range
+template<typename String>
+bool str_is_ascii(String const& s) {
+	for (auto const& c : s) {
+		if (static_cast<std::make_unsigned_t<typename String::value_type>>(c) > 127) {
+			return false;
+		}
+	}
+
+	return true;
+}
+
+/// \brief Return passed string with all leading and trailing whitespace removed
+template<typename String>
+String trimmed(String const& s) {
+	size_t const first = s.find_first_not_of(fzS(typename String::value_type, " \r\n\t"));
+	if (first == String::npos) {
+		return String();
+	}
+	else {
+		// Cannot be npos
+		size_t const last = s.find_last_not_of(fzS(typename String::value_type, " \r\n\t"));
+
+		return s.substr(first, last - first + 1);
+	}
+}
+
+
+/// \brief Remove all leading and trailing whitespace from string
+template<typename String>
+void trim(String & s) {
+	s = trimmed(s);
+}
+
 }
 
 #endif
